@@ -266,10 +266,12 @@ func (c *ConnManager) setConn(
 
 	existingConn, ok := c.connections[ck]
 	if ok {
-		conn.session.EndSession(context.Background())
-		conn.session.Client().Disconnect(context.Background()) //nolint:errcheck,gosec
+		err := closeSession(context.Background(), conn.session)
+		if err != nil {
+			c.log.Warningf("set conn session client clean-up failed: %s", err.Error())
+		}
 
-		Impl.Debugf("Closed unused connection: %s", ck.uri.Addr())
+		c.log.Debugf("Closed unused connection: %s", ck.uri.Addr())
 
 		return existingConn
 	}
@@ -286,10 +288,13 @@ func (c *ConnManager) closeUnused() {
 
 	for ck, conn := range c.connections {
 		if time.Since(conn.lastTimeAccess) > c.keepAlive {
-			conn.session.EndSession(context.Background())
-			_ = conn.session.Client().Disconnect(context.Background())
+			err := closeSession(context.Background(), conn.session)
+			if err != nil {
+				c.log.Warningf("unused session client clean-up failed: %s", err.Error())
+			}
+
 			delete(c.connections, ck)
-			Impl.Debugf("Closed unused connection: %s", ck.uri.Addr())
+			c.log.Debugf("Closed unused connection: %s", ck.uri.Addr())
 		}
 	}
 }
@@ -298,12 +303,15 @@ func (c *ConnManager) closeUnused() {
 func (c *ConnManager) closeAll() {
 	c.connectionsMu.Lock()
 	for uri, conn := range c.connections {
-		conn.session.EndSession(context.Background())
-		_ = conn.session.Client().Disconnect(context.Background())
+		err := closeSession(context.Background(), conn.session)
+		if err != nil {
+			c.log.Warningf("close all session client clean-up failed: %s", err.Error())
+		}
+
 		delete(c.connections, uri)
 	}
 	c.connectionsMu.Unlock()
-	Impl.Debugf("Closed all connections")
+	c.log.Debugf("Closed all connections")
 }
 
 // housekeeper repeatedly checks for unused connections and close them.
@@ -345,15 +353,29 @@ func (c *ConnManager) create(
 
 	session, err := client.StartSession()
 	if err != nil {
+		c.log.Debugf("session start failed failed: %s", ck.uri.Addr())
+
+		cerr := closeSession(context.Background(), session)
+		if cerr != nil {
+			c.log.Warningf("session start clean-up failed: %s", cerr.Error())
+		}
+
 		return nil, err
 	}
 
 	err = session.Client().Ping(context.Background(), readpref.Nearest())
 	if err != nil {
+		c.log.Debugf("session client ping failed: %s", ck.uri.Addr())
+
+		cerr := closeSession(context.Background(), session)
+		if cerr != nil {
+			c.log.Warningf("session client ping clean-up failed: %s", cerr.Error())
+		}
+
 		return nil, err
 	}
 
-	Impl.Debugf("Created new connection: %s", ck.uri.Addr())
+	c.log.Debugf("Created new connection: %s", ck.uri.Addr())
 
 	return &MongoConn{
 		addr:           ck.uri.Addr(),
@@ -505,6 +527,17 @@ func (conn *MongoConn) checkConnection() {
 	}
 }
 
+func closeSession(ctx context.Context, session mongo.Session) error {
+	session.EndSession(ctx)
+
+	err := session.Client().Disconnect(ctx)
+	if err != nil {
+		return errs.Wrapf(err, "failed to close session client connection")
+	}
+
+	return nil
+}
+
 func createConnKey(uri uri.URI, params map[string]string) connKey {
 	tlsType := params[tlsConnectParam]
 	if tlsType == "" {
